;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Giant Jumping Piranha Plant
;;   by TheBiob
;; Description: This is basically original smw's Jumping Piranha plant just with a different gfx routine and hitbox.
;;
;; Uses first extra bit: YES:
;;   If extra bit is set it'll spit fire.
;; Original disassembly by imamelia
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;
;   TABLES    ;
;;;;;;;;;;;;;;;

Frame1:    db $00,$10,$00,$10 ; Closed mouth tiles
Frame2:    db $02,$12,$02,$12 ; Open mouth tiles
YXPCCC:    db $09,$09,$49,$49 ; YXPPCCCT properties
XPosOff:   db $04,$04,$0C,$0C ; X position offset
YPosOff:   db $00,$08,$00,$08 ; Y position offset

StemFrame1:  db $30,$31,$30 ; Stem tiles 1
StemFrame2:  db $32,$33,$32 ; Stem tiles 2
StemYXPCCC:  db $0B,$0B,$4B ; YXPPCCCT properties
StemXPosOff: db $04,$0C,$14 ; X position offset
StemYPosOff: db $18,$18,$18 ; Y position offset

;;;;;;;;;;;;;;;
; Sprite INIT ;
;;;;;;;;;;;;;;;

	print "INIT ",pc
		; Sprite INIT code
	RTL
	
;;;;;;;;;;;;;;;
; Sprite MAIN ;
;;;;;;;;;;;;;;;

	print "MAIN ",pc
		PHB : PHK : PLB
			JSR SpriteMain
		PLB
	RTL
; Original disassembly code. Pretty much not edited at all except for the graphic part at the top and the JSL $0086DF part
SpriteMain:

JSR JumpingPiranhaGFX1

LDA $9D			;
BNE Return00		; return if sprites are locked

LDY #$00
JSR SubOffScreen		;
JSL $01803A		; interact with the player and with other sprites
JSL $01801A		; update sprite Y position without gravity

LDA $C2,x		; sprite state
PHX			; \ Jump to next state
ASL			; |
TAX			; |
REP #$20	; |
LDA Pntr,x	; |
STA $0A		; |
SEP #$20	; |
PLX			; |
JMP ($000A)	; /
;LDA $C2,x
;JSL $0086DF		; execute 16-bit pointer subroutine
Pntr:
dw State00		; Pointer to the different parts of the sprite
dw State01		;
dw State02		;

Return00:			;
RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; code for sprite state 00
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

State00:

STZ $AA,x	; set the sprite Y speed to 0
LDA $1540,x	;
BNE Return01	;

JSR SubHorzPos	;

LDA $0F		; horizontal distance between the sprite and the player
CLC		;
ADC #$1B		;
CMP #$37		; if the player is close enough...
BCC Return01	;

LDA #$C0		; set the sprite's jumping Y speed
STA $AA,x	;
INC $C2,x	; increment the sprite state
STZ $1602,x	; and set the frame to 0

Return01:		;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; code for sprite state 01
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

State01:

LDA $AA,x	; if the sprite's Y speed is negative...
BMI Accelerate	;
CMP #$40		; or it is positive and less than 40...
BCS NoAccelerate	;
Accelerate:	;
CLC		;
ADC #$02	; add 2 to it to make the sprite accelerate
STA $AA,x	;

NoAccelerate:	;

INC $1570,x	; increment the animation frame counter
LDA $AA,x	; if the sprite's Y speed
CMP #$F0		; is between F1 and FF...
BMI Return02	;
LDA #$50		;
STA $1540,x	; set the timer
INC $C2,x	; and increment the sprite state

RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; code for sprite state 02
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

State02:

INC $151C,x	; increment the secondary animation frame counter
LDA $1540,x	; if the timer is set, then skip the next part of code
BNE MaybeSpitFire	; and go to the part where it checks to see whether or not the sprite should spit fireballs

NoFire:		;

INC $1570,x	;

LDA $14		;
AND #$03	; once every 4 frames...
BNE NoAccelerate2	;
LDA $AA,x	;
CMP #$08		; if the sprite Y speed is greater than 08...
BPL NoAccelerate2	;
INC		;
STA $AA,x	; increment the sprite Y speed

NoAccelerate2:	;

JSL $019138	; interact with objects

LDA $1588,x	;
AND #$04	; if the sprite is on the ground...
BEQ Return02	;
STZ $C2,x	; reset the sprite state to 0
LDA #$40		;
STA $1540,x	; and set the time before it checks the player's proximity again

Return02:		;
RTS

MaybeSpitFire:

LDA $7FAB10,x	;
AND #$04	; if the extra bit is not set...
BEQ NoFire	; don't spit fireballs

STZ $1570,x	; reset the animation frame counter

LDA $1540,x	; I had to reload this because of the extra bit check.
CMP #$40		; if the timer isn't at exactly 40...
BNE NoFire	;
LDA $15A0,x	; or the sprite is offscreen horizontally or vertically...
ORA $186C,x	;
BNE NoFire	; don't spit fireballs

LDA #$10		; X speed for the first fireball
JSR SpitFireballs	;
LDA #$F0		; X speed for the second fireball

SpitFireballs:	; It's kind of interesting how they did this, really.

STA $00		; save the X speed value

LDY #$07		; 8 extended sprite indexes to loop through

ExSpriteLoop:	;

LDA $170B,y	; check this slot
BEQ SpawnFire	; if the slot is free, we can use it for a fireball
DEY		; if not, decrement the index
BPL ExSpriteLoop	; and try again

RTS		;

SpawnFire:	;

LDA #$0B		; extended sprite number = 0B
STA $170B,y	; piranha fireball

LDA $E4,x	; sprite X position
CLC		;
ADC #$0C	; the fireball spawns C pixels to the right of the plant
STA $171F,y	;
LDA $14E0,x	;
ADC #$00	; prevent overflow
STA $1733,y	;

LDA $D8,x	;
STA $1715,y	; same Y position as the plant
LDA $14D4,x	;
STA $1729,y	;

LDA #$D0	; extended sprite Y speed = D0
STA $173D,y	;
LDA $00		; $00 had the X speed value from before
STA $1747,y	;

BRA NoFire	; finish up with the regular code
RTS
;; Here is the edited graphic routine:
;; Basically just looping through the tables at the top
;; Draw sprite:
JumpingPiranhaGFX1:
	JSR GetDrawInfo ; > Get information
	
	JSR Head	; \ Draw head
	JSR Stem	; / Draw stem
	
	LDX $15E9	; Load current sprite index
	
	LDY #$FF		; 00 = 8x8, 02 = 16x16, FF = Don't set tile size (Already set)
	LDA #$06		; How many tile have been drawn - 1
	JSL $01B7B3		; reserve
RTS

Head:
	LDA $1570,x	; \ Get animation counter into $03 since X won't be the sprite's slot any more
	STA $03		; /
	LDX #$03 ; GFX loop counter
-	LDA $00			; x position
	CLC : ADC XPosOff,x
	STA $0300,y
	LDA $01			; y position
	CLC : ADC YPosOff,x
	STA $0301,y
	LDA $03
	AND #$10
	BEQ +
	LDA Frame1,x
	BRA ++
+	LDA Frame2,x
++	STA $0302,y
	LDA YXPCCC,x
	STA $0303,y
	PHY
	TYA : LSR : LSR : TAY	; \ Set size to 16x16
	LDA #$02				; |
	STA $0460,y				; |
	PLY						; /
	INY : INY	; \ Next OAM slot
	INY : INY	; /
	DEX : BPL -
RTS

Stem:
	LDX #$02 ; GFX loop counter
-	LDA $00			; x position
	CLC : ADC StemXPosOff,x
	STA $0300,y
	LDA $01			; y position
	CLC : ADC StemYPosOff,x
	STA $0301,y
	LDA $14
	AND #$08
	BEQ +
	LDA StemFrame1,x
	BRA ++
+	LDA StemFrame2,x
++	STA $0302,y
	LDA StemYXPCCC,x
	STA $0303,y
	PHY
	TYA : LSR : LSR : TAY	; \ Set size to 8x8
	LDA #$00				; |
	STA $0460,y				; |
	PLY						; /
	INY : INY	; \ Next OAM slot
	INY : INY	; /
	DEX : BPL -
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GetDrawInfo - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GetDrawInfo:
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$20
		SEC
		SBC $1A
		STA $00
		CLC
		ADC.w #$0040
		CMP.w #$0180
		SEP #$20
		LDA $01
		BEQ +
		LDA #$01
+		STA $15A0,x
		TDC
		ROL A
		STA $15C4,x
		BNE .Invalid

		LDA $14D4,x
		XBA
		LDA $1662,x
		AND #$20
		BEQ .CheckOnce
.CheckTwice	LDA $D8,x
		REP #$21
		ADC.w #$001C
		SEC
		SBC $1C
		SEP #$20
		LDA $14D4,x
		XBA
		BEQ .CheckOnce
		LDA #$02
.CheckOnce	STA $186C,x
		LDA $D8,x
		REP #$21
		ADC.w #$000C
		SEC
		SBC $1C
		SEP #$21
		SBC #$0C
		STA $01
		XBA
		BEQ .OnScreenY
		INC $186C,x
.OnScreenY	LDY $15EA,x
		RTS
		
.Invalid	PLA
		PLA
		RTS
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubOffScreen - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
SubOffScreen:	LDA $15A0,x
		ORA $186C,x
		BEQ .Return2
		LDA $167A,x
		AND #$04
		BNE .Return2
		LDA $5B
		LSR A
		BCS .VerticalLevel
		LDA $D8,x
		ADC #$50
		LDA $14D4,x
		ADC #$00
		CMP #$02
		BCS .EraseSprite
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$21
		ADC .AddTable,y
		SEC
		SBC $1A
		CMP .CmpTable,y
.Common		SEP #$20
		BCC .Return2
.EraseSprite	LDA $14C8,x
		CMP #$08
		BCC .KillSprite
		LDY $161A,x
		CPY #$FF
		BEQ .KillSprite
		LDA $1938,y
		AND #$FE
		STA $1938,y
.KillSprite	STZ $14C8,x
.Return2	RTS

.VerticalLevel	LDA $13
		LSR A
		BCS .CheckY
		LDA $14E0,x
		XBA
		LDA $E4,x
		REP #$21
		ADC.w #$0040
		CMP.w #$0280
		BRA .Common
.CheckY		LDA $14D4,x
		XBA
		LDA $D8,x
		REP #$20
		SBC $1C
		CLC
		ADC.w #$0070
		CMP.w #$01D0
		BRA .Common

.AddTable	dw $0040,$0040,$0010,$0070
		dw $0090,$0050,$0080,$FFC0
.CmpTable	dw $0170,$01E0,$01B0,$01D0
		dw $0230,$01B0,$0220,$0160
		
SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS
